@@ -64,7 +64,7 @@
 :- autoload(library(prolog_codewalk),
 	    [prolog_walk_code/1,prolog_program_clause/2]).
 :- autoload(library(prolog_format),[format_types/2]).
-
+:- autoload(library(predicate_options), [check_predicate_options/0]).
 
 :- set_prolog_flag(generate_debug_info, false).
 
@@ -101,11 +101,32 @@ predicates in `user' modules.
 %     * list_undefined/0 reports undefined predicates
 %     * list_trivial_fails/0 reports calls for which there is no
 %       matching clause.
+%     * list_format_errors/0 reports mismatches in format/2,3
+%       templates and the list of arguments.
 %     * list_redefined/0 reports predicates that have a local
 %       definition and a global definition.  Note that these are
-%       *not* errors.
+%       __not__ errors.
+%     * list_void_declarations/0 reports on predicates with defined
+%       properties, but no clauses.
 %     * list_autoload/0 lists predicates that will be defined at
 %       runtime using the autoloader.
+%     * check_predicate_options/0 tests for options passed to
+%       predicates such as open/4 that are unknown or are used
+%       with an invalid argument.
+%
+%    The checker can be expanded or  restricted by modifying the dynamic
+%    multifile hook checker/2.
+%
+%    The checker may be used in batch, e.g., for CI workflows by calling
+%    SWI-Prolog as below. Note that by using ``-l`` to load the program,
+%    the program is not started  if   it  used  initialization/2 of type
+%    `main` to start the program.
+%
+%
+%    ```
+%    swipl -q --on-warning=status --on-error=status \
+%          -g check -t halt -l myprogram.pl
+%    ```
 
 check :-
     checker(Checker, Message),
@@ -847,12 +868,13 @@ valid_string_goal(codesio:format_to_codes(Format,_,_,_)) :- string(Format).
 %      retract(check:checker(list_redefined,_)).
 %      ```
 
-checker(list_undefined,         'undefined predicates').
-checker(list_trivial_fails,     'trivial failures').
-checker(list_format_errors,     'format/2,3 and debug/3 templates').
-checker(list_redefined,         'redefined system and global predicates').
-checker(list_void_declarations, 'predicates with declarations but without clauses').
-checker(list_autoload,          'predicates that need autoloading').
+checker(list_undefined,          'undefined predicates').
+checker(list_trivial_fails,      'trivial failures').
+checker(list_format_errors,      'format/2,3 and debug/3 templates').
+checker(list_redefined,          'redefined system and global predicates').
+checker(list_void_declarations,  'predicates with declarations but without clauses').
+checker(list_autoload,           'predicates that need autoloading').
+checker(check_predicate_options, 'predicate options lists').
 
 
                  /*******************************
